#!/bin/sh
#
# notify.sh - host and service notification command for OP5 Monitor
#
# This should be GPL'ed and delivered 'as is', or at least with most of its
# functionality retained.
#
# Author: Andreas Ericsson <ae@op5.se>
#
# Copyright (C) 2004 OP5 AB
# All rights reserved.
#

## some files we have to use
config_dir="/opt/monitor/etc"
var_dir="/opt/monitor/var"
nagios_cfg="$config_dir/nagios.cfg"
cgi_cfg="$config_dir/cgi.cfg"

# this is removed in the 'trap' later on.
tmpf=`mktemp /tmp/notify.sh.tmp.XXXXXXXXXXX`

# determine comment_file and url_html_path
cat $cgi_cfg $nagios_cfg 2>&1 | grep -e comment_file -e url_html_path |\
	sed -e s/^\ \*//g | grep -v -e ^$ -e ^# > $tmpf
source $tmpf

test -z "$comment_file" && comment_file="$var_dir/comment.log"
test -z "$url_html_path" && url_html_path="/monitor"
# make sure url_html_path starts with a '/'
test "`echo $url_html_path | cut -b1`" = "/" || \
	url_html_path="/${url_html_path}"

# for debugging pusposes
notify_log_file="$var_dir/notify.log"

# thses will contain the text that actually gets sent
msg_mail=`mktemp /tmp/notification_mail.XXXXXXX`
msg_sms=`mktemp /tmp/notification_sms.XXXXXXX`

# hardcode this if it doesn't seem to work for you
#status_url="https://monitor.op5.se/monitor/cgi-bin/status.cgi"
hostname=`hostname -f`
test -z "$hostname" && hostname=`hostname`
status_url="https://${hostname}${url_html_path}/cgi-bin/status.cgi"

# mail_cmd must accept subject as an argument and must be able to read
# mail body from stdin.
mail_cmd="/bin/mail -s "

# sms_cmd must accept a cell-phone number as argument, and must be
# able to read message from stdin.
sms_cmd="/opt/bin/sendsms"

## for action decisions
ARGS_HOST_LIMIT=12
ARGS_SERVICE_LIMIT=15
ARGS_HIGH_LIMIT=22
ack="ACKNOWLEDGEMENT"

ARGS=$#

### (This is where the errors happen)
## count off the unset arguments
for i in "$@" ; do test -z "$i" && ARGS=$((ARGS - 1)) ; done

trap "rm -f $msg_mail $msg_sms $tmpf" EXIT INT
test $? -gt 0 && echo "UNABLE TO 'trap'"

ARGS=$#

###############################################################################
#                          functions code block
###############################################################################

function usage () {
cat << EOMSG
usage: ${0##*/} ARG1 ARG2 ... ARGn

You should never call this script by hand. Instead, you should set it up to
be the default notification command for Nagios.

Syntax:
The NUMBER of arguments decide what kind of notification we should send out.
Note that some macros may be set to zero length strings, while some others
may contain spaces, but they will still have to be counted properly.
For this to work, the macros given on the command_line for this command
object statement MUST be encased in double quotes, like so;

define command{
    command_name   host_notify
    command_line   notify.sh "\$CONTACTEMAIL\$" "\$CONTACTPAGER\$" (and so on...)
    }

If argc >= $ARGS_HOST_LIMIT, it is considered to be a host state notification.
If argc >= $ARGS_SERVICE_LIMIT, it is considered to be a service state notification.

EOMSG
	exit 0
}

###############################################################################
#                            variable setup
###############################################################################

test $ARGS -lt $ARGS_HOST_LIMIT && usage
test $ARGS -gt $ARGS_HIGH_LIMIT && usage

## contact info
contact_name="${1}"
contact_email="${2}"
contact_pager="${3}"

## host info
host_name="${4}"
host_alias="${5}"
host_address="${6}"
host_state="${7}"

## miscellaneous info
# Strip html from output
output=`echo "${8}" | sed "s/<[^>|.]*>//g"`
execution_time="${9}"
date="${10}"
time="${11}"

## notification type info
notification_type="${12}"
notification_number="${13}"

# overwritten if service alert
subject="$notification_type: '$host_name' is $host_state"

## only for service alerts
if [ $ARGS -ge $ARGS_SERVICE_LIMIT ]; then
	service_desc="${14}"
	service_state="${15}"
	check_latency="${16}"
	subject="$notification_type: '$service_desc' on '$host_name' is $service_state"
fi

###############################################################################
#                            main code block
###############################################################################

## logging and debugging
#echo "To: $contact_name <$contact_email>" >> $notify_log_file
#echo "Subject: $subject" >> $notify_log_file
#echo "notificaction_type=$notification_type" >> $notify_log_file

# for nilex / ARS / anything integration
# if there's a directory in /opt/monitor/etc by the name of the user, we
# superimpose .alert_types and any variables found in the directory.
# grep for the host_name. If not found; exit. otherwise, strip junk and
# print to file. Must grep case insensitive (Windows thingie, I suppose)
if [ -d /opt/monitor/etc/$contact_name ]; then
	old_pwd=`pwd`
	cd /opt/monitor/etc/$contact_name
	grep "$notification_type" .notification_types >& /dev/null
	if [ $? -gt 0 -a -f ".notification_types" ]; then
		echo "$contact_name doesn't want $notification_type notifications" >> $notify_log_file
		exit 0
	fi
	for x_var in * ; do
		x_val=`grep -i ^$host_name: $x_var | sed s/^.*://`
		if ! [ -z "$x_val" ]; then
			echo -e "$x_var=$x_val\n" >> $msg_mail
		fi
		unset x_var
		unset x_val
	done
	cd $old_pwd
fi

echo -e "OP5 Monitor\n" >> $msg_mail
# not acknowledgement
if ! [ "$notification_type" = "$ack" ]; then
	if [ $ARGS -ge $ARGS_SERVICE_LIMIT ]; then
		echo "Service ${notification_type} detected ${date} ${time}." >> $msg_mail
	else
		echo "Host ${notification_type} detected ${date} ${time}." >> $msg_mail
	fi
else
	# it's an acknowledgement, so get acknowledging username from comment_file
	if [ $ARGS -ge $ARGS_SERVICE_LIMIT ]; then
		acking_user=`grep ACKNOWLEDGEMENT $comment_file |\
					cut -d\; -f3,6 | grep ^$host_name | tail -n 1 |\
					cut -d\; -f2`
	else
		acking_user=`grep ACKNOWLEDGEMENT $comment_file |\
					cut -d\; -f3,5 | grep ^$host_name | tail -n 1 |\
					cut -d\; -f2`
	fi
	test -z "$acking_user" && acking_user="Someone"
fi

# general
if [ $ARGS -ge $ARGS_SERVICE_LIMIT ]; then
	echo "'$service_desc' on host '$host_name' has passed the $service_state threshhold." >> $msg_mail
else
	echo "Host '$host_name' ($host_address) is $host_state." >> $msg_mail
fi

test "$notification_type" = "$ack" && \
	echo -e "$acking_user acknowledged this problem $date $time." >> $msg_mail

# produce the link
echo "${status_url}?host=${host_name}" >> $msg_mail
echo >> $msg_mail

echo "Additional info;" >> $msg_mail
echo "$output" >> $msg_mail
echo >> $msg_mail

cat << EOF >> $msg_mail
Host   : $host_name
Address: $host_address
Alias  : $host_alias
Status : $host_state
EOF

if [ $ARGS -ge $ARGS_SERVICE_LIMIT ]; then
	echo >> $msg_mail
	echo "Service: $service_desc" >> $msg_mail
	echo "Status : $service_state" >> $msg_mail
	if ! [ $notification_type = $ack ]; then
		if [ $check_latency -gt 0 ]; then
			echo "Latency: Check was $check_latency seconds behind schedule" >> $msg_mail
		else
			echo "Latency: Check was performed on schedule" >> $msg_mail
		fi
		echo "Misc   : Check took $execution_time seconds to complete" >> $msg_mail
	fi
fi

# only send if $contact_mail is set
test -z "$contact_email" || cat $msg_mail | $mail_cmd "$subject" $contact_email

## debug stuff
#echo "Sent to $contact_name ($contact_email)" >> $log_mail
#echo "Subject: $subject" >> $log_mail
#cat $msg_mail >> $log_mail
#echo "--------------------------------------------" >> $log_mail

#############################################################################
## sms
#############################################################################

# exit if we're not supposed to send it.
test -z "$contact_pager" && exit 0

echo -n "${notification_type}: " >> $msg_sms
if [ $ARGS -ge $ARGS_SERVICE_LIMIT ]; then
	echo -n "$service_state alert from " >> $msg_sms
	echo -n "'$service_desc' on '$host_name' ($host_address)" >> $msg_sms
else
	echo -n "'$host_name' ($host_address) is $host_state. " >> $msg_sms
fi

# last thing, so we add the new-line.
echo "$output" >> $msg_sms

head -c 160 $msg_sms | $sms_cmd "$contact_pager"
